;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#xee2ecca30194fd75479e0fcdfd2b53b7 . "./code.cg2js/cg-acache.fasl")
(#xa1a1fa83f2301934ab9a3530456ebd09 . "./code.cg2js/cg-base.fasl")
(#x8b123a5a01cd86a9a6596ab564e95650 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x68e6013d542eb190381ba85a061ade18 . "./code.cg2js/cg-bitmap-pane.fasl")
(#xa6343fabfebc2deee56eebe0612451cc . "./code.cg2js/cg-bitmap-stream.fasl")
(#xd20ec695275d53fabd04639fac976133 . "./code.cg2js/cg-button.fasl")
(#xa57f113d42d1afa205bc53ee2f156752 . "./code.cg2js/cg-calendar.fasl")
(#x6615ac7519a8546cd88a9f7979958f9e . "./code.cg2js/cg-caret.fasl")
(#xa29baff7eab4b74e6adf8dac9a6be1b9 . "./code.cg2js/cg-chart-or-plot.fasl")
(#x62f6fd79bf60b3155d9ae0683644df67 . "./code.cg2js/cg-chart-widget.fasl")
(#x9d382cec719f73b3ab86509151e6ffa1 . "./code.cg2js/cg-check-box.fasl")
(#x9f6e32ac0433f201d7361a7b34b44d54 . "./code.cg2js/cg-choice-list.fasl")
(#xcefb230aa27d7a12d65d94eb1f093072 . "./code.cg2js/cg-choose-printer.fasl")
(#x6c06af59a76acbbbe3a7ede39b223995 . "./code.cg2js/cg-class-grid.fasl")
(#xc5933cb95b88e47fa4cdc8e596261f9a . "./code.cg2js/cg-class-slot-grid.fasl")
(#x45859b7c94f62a3c50ade070e57f54ce . "./code.cg2js/cg-class-support.fasl")
(#xefb461e4cfce8bb5a62ec8febff32650 . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#xb45abddc423085f1acdc7f80c45afe74 . "./code.cg2js/cg-clipboard-stack.fasl")
(#x5b9e22be0fb895403b16fcd7aef51b0c . "./code.cg2js/cg-clipboard.fasl")
(#xb89d334d2ffdbfcfed21daa7ab4e7143 . "./code.cg2js/cg-color-dialog.fasl")
(#xfb6af2a0a4cd308b8396e0d5590b2ced . "./code.cg2js/cg-combo-box.fasl")
(#xe67abac529804689f60d3c2a4290ed33 . "./code.cg2js/cg-common-control.fasl")
(#xdf458fea573856a68ad17314bd20b854 . "./code.cg2js/cg-comtab.fasl")
(#x18e215998916d0b4b76a02afd8958623 . "./code.cg2js/cg-cursor-pixmap.fasl")
(#x8554df598b839ee4918c6bd5d0be54a1 . "./code.cg2js/cg-curve.fasl")
(#xe68e8afd2dbed36a6fecf06fc55b16f4 . "./code.cg2js/cg-dde-utils.fasl")
(#x6a66b019de11a53651b2479f43e2e2ce . "./code.cg2js/cg-dev.fasl")
(#xf4e0b91aa809a4bdbf4b315846716106 . "./code.cg2js/cg-dialog-item.fasl")
(#xca65a7818bdb691d349df7c5b51e8ee8 . "./code.cg2js/cg-directory-dialog-os.fasl")
(#xe47b3b3fda3f5958297ec69024d4ab3b . "./code.cg2js/cg-directory-dialog.fasl")
(#xc79554134857e70f928ff4a8b6adf8c7 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#x35348042d9ea1b0ae002bf6cc2425b6f . "./code.cg2js/cg-drag-and-drop.fasl")
(#xe9a82d0770e56a3bea6791c565be67d9 . "./code.cg2js/cg-drawable-clipboard.fasl")
(#xa717c417462a776ed3da0a1087c7baaa . "./code.cg2js/cg-drawable.fasl")
(#x080adb427283323acbe801933b9c496e . "./code.cg2js/cg-dropping-outline.fasl")
(#x72c3a1774a84e46baa3423086883a0ab . "./code.cg2js/cg-edit-in-place.fasl")
(#x206e69375c8765b933112a0ad833cf4b . "./code.cg2js/cg-editable-text.fasl")
(#x21eee7ddeaa4cd192fc0d38c921e0f0c . "./code.cg2js/cg-file-dialog.fasl")
(#xa2e441f891ed25dffcf1396403aee31b . "./code.cg2js/cg-fill-texture.fasl")
(#x2fa87b57758a1cef3da0f5d50ccefb41 . "./code.cg2js/cg-find-string-dialog.fasl")
(#xc31302f5bc567ec0b4af8967efbacb55 . "./code.cg2js/cg-font-dialog.fasl")
(#x13ef1df1ef57b55ef71490b8a05b3caa . "./code.cg2js/cg-gesture-emulation.fasl")
(#xe11027e56fe794db4a25bf87b0098e87 . "./code.cg2js/cg-get-pixmap.fasl")
(#x4f033308fe54d15f3dd5f57f1391efc9 . "./code.cg2js/cg-get-position.fasl")
(#x68094b96df7a34cbe716303386ebf805 . "./code.cg2js/cg-graphics-context.fasl")
(#x22f04da09a270f88479317dc92600170 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#x9bb235da0204af6af6c9013d9da0f5eb . "./code.cg2js/cg-grid-widget.fasl")
(#xb955d53e8e8e67671ac476e2d0b1ede7 . "./code.cg2js/cg-group-box.fasl")
(#x5ec6c3725bb3145c03000261615bbe71 . "./code.cg2js/cg-header-control.fasl")
(#xb190f0831f47ac2f9485ef34951c81ea . "./code.cg2js/cg-hotspot.fasl")
(#x0526fa5f93b91cd444acb93e2b8ba5b3 . "./code.cg2js/cg-html-dialog.fasl")
(#x61b871de57f98f782754601281497111 . "./code.cg2js/cg-html-widget.fasl")
(#xe8f987cc4f36173d27c0140ad51c4418 . "./code.cg2js/cg-icon-pixmap.fasl")
(#x15ba8dbf0ba49bc11b1bb7e6bd0bfd6f . "./code.cg2js/cg-icon.fasl")
(#xbe6222ea0a1c7b35677a6322e9e56222 . "./code.cg2js/cg-ie.fasl")
(#x2e26729dc5e3938734ea029b29c51c55 . "./code.cg2js/cg-intersect-posbox.fasl")
(#x4715cac786a33679db06f660894cee66 . "./code.cg2js/cg-intersect.fasl")
(#x71693724fffa6e9f1df7981d5dd0df6b . "./code.cg2js/cg-item-list.fasl")
(#xd1748528e3760d0b2600909a24e68795 . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x02f805225a59fc8fb7be079229806cf5 . "./code.cg2js/cg-lamp.fasl")
(#x8f64053f5736062765aaf83b23f86656 . "./code.cg2js/cg-layout.fasl")
(#x8be7ad7d39a4e93ba8c423257de74be8 . "./code.cg2js/cg-lettered-menu.fasl")
(#x64fb55dacbf60234764f81727395f220 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x4fdbc5fc33cca17f1352efe7dfea7710 . "./code.cg2js/cg-lisp-text.fasl")
(#x2c5e8ccea5c0653ac0503469737ce8e4 . "./code.cg2js/cg-lisp-widget.fasl")
(#xaf0eab424fbcea6130cfcf352731dac4 . "./code.cg2js/cg-list-view.fasl")
(#x1555217563820986691a48f714003ef6 . "./code.cg2js/cg-mci.fasl")
(#xd22fdf1c968e4e78a688837daab249ff . "./code.cg2js/cg-menu-tooltip.fasl")
(#xe231d3e6c7cde07e5637a0948063473a . "./code.cg2js/cg-menu.fasl")
(#x22ceab427d99c5efac1a3f6bf43b5492 . "./code.cg2js/cg-message-dialog.fasl")
(#xc373a616c343100d0f9e5969bb4dc40e . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#xedc818b8b6b225673d3b4e572f52ccea . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x2ed9d157936f7eba5a27d0b1c16cc9e8 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#x500e063c659db594853c132008b0d323 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#xf72a330507c25474ec6b93dd5676301d . "./code.cg2js/cg-multi-picture-button.fasl")
(#x1e6356738e29097dded2a26fbd34306e . "./code.cg2js/cg-nodes.fasl")
(#x4ce70e7db717bfee380bd26e53ae0ca9 . "./code.cg2js/cg-object-editor-layout.fasl")
(#xd094ccfd77b4eb1fda3b263e5eb83f53 . "./code.cg2js/cg-object-editor.fasl")
(#xc4d652e92c8d0b02ab074c5784c5321d . "./code.cg2js/cg-ocx.fasl")
(#xcd779e0ff401cf22efe5ee956c8b3709 . "./code.cg2js/cg-os-widget.fasl")
(#xdb8bc5371a926869129095189f5d0156 . "./code.cg2js/cg-os-window.fasl")
(#x8259f849655106ff614c3e3c969b58c5 . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#xd0ca0fed5c63d1cf5813dc7673c101f8 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#x8c5f4c02bbb661a06a9333f7fd0d4dc5 . "./code.cg2js/cg-outline.fasl")
(#x4831b72a8328fcee68300f558396812d . "./code.cg2js/cg-palette.fasl")
(#x0a2eebe760199e28e94097daa2eafdfe . "./code.cg2js/cg-paren-matching.fasl")
(#x8da3dc50a1fa859af5bec6933e7750fc . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x0ad9e5a5e316cd859bbc0de6644ced5c . "./code.cg2js/cg-picture-widget.fasl")
(#xba21babc533267269643abdaa51531a1 . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x8acbf4c75a426358a9a301af1c586e39 . "./code.cg2js/cg-pixmap-printing.fasl")
(#xd6262971ce361503bc2f42340f65afc9 . "./code.cg2js/cg-pixmap-rotate.fasl")
(#xbce1cc658bcaad0a7fb4735aaa6bcddc . "./code.cg2js/cg-pixmap-widget.fasl")
(#xf2e60695be68b8ce45368d9e19a64580 . "./code.cg2js/cg-pixmap.fasl")
(#x9119be43ce6e4ddef6c79be063a1900d . "./code.cg2js/cg-player.fasl")
(#xc2a0c98f5f3dfe46921c8ca5ab89ade1 . "./code.cg2js/cg-plot-widget.fasl")
(#x345c93feceeefa2cc499da3380fff95a . "./code.cg2js/cg-printing.fasl")
(#xf805acc936dc17fab25dce8bd62c78b2 . "./code.cg2js/cg-progress-indicator.fasl")
(#x5be9309d0dbdbc2533acb73bde944b98 . "./code.cg2js/cg-project-window.fasl")
(#xb21d71f6c062cc73d3db2e12e242f1f8 . "./code.cg2js/cg-property.fasl")
(#x0549194e3e52e818fc61b7bd2e7a8aba . "./code.cg2js/cg-radio-button.fasl")
(#xb97b00e042f76cd33da2d73e015c17df . "./code.cg2js/cg-rich-edit-pane-clipboard.fasl")
(#xf65418a6b0b907595c6ad012b0b45d5e . "./code.cg2js/cg-rich-edit-pane-printing.fasl")
(#xf7ce5b898c02c7b4eb34bbc1e6c01372 . "./code.cg2js/cg-rich-edit-pane.fasl")
(#x53d34a9eecca6877371bb3aecdc3be26 . "./code.cg2js/cg-rich-edit.fasl")
(#x0be98d585e55fbffb435b6200ec39130 . "./code.cg2js/cg-sample-file-menu.fasl")
(#xc394c89f8788b6018c2ccff64ad97b92 . "./code.cg2js/cg-scaling-stream.fasl")
(#xc630f7b444f70b675a7d04d114a6dabe . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#xae92d924d8efc065c72be47fe1ae3aa9 . "./code.cg2js/cg-scroll-bar.fasl")
(#x0033f3e89e44b3ee7a1a054cb1140777 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#x75d2b66d7f14d6fa23f20802a9aead48 . "./code.cg2js/cg-selected-object.fasl")
(#xe6cbb726e9d45bee954fc1a4a5679bdb . "./code.cg2js/cg-shortcut-menu.fasl")
(#x190b1e37176edc9bd74ba60036003551 . "./code.cg2js/cg-split-bar.fasl")
(#x673347c6679ef49074985560c3d012c8 . "./code.cg2js/cg-static-text.fasl")
(#x67e4446901055611f8f7f6a07ffe3dae . "./code.cg2js/cg-status-bar.fasl")
(#x1dfe34d7c2bc6c55e4c742f3b4bca7ce . "./code.cg2js/cg-string-dialog.fasl")
(#x4e99d0f92fa9af8f71de47cd97968513 . "./code.cg2js/cg-tab-control.fasl")
(#x85b15032756a8426416c70238e90dfad . "./code.cg2js/cg-template-string.fasl")
(#x7833568f6e11ad3cf7713dd2fa234e4d . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#x5f62b4fb1c141ee99ea0cd7226d045a1 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#x05733cd6fd31e670f5c1bfdaa055ad65 . "./code.cg2js/cg-text-edit-pane.fasl")
(#xafc65d24b254764617622d8ab9a898c3 . "./code.cg2js/cg-text-or-combo.fasl")
(#x3dbcaf76e6d38db989c99c7b1d4072ee . "./code.cg2js/cg-text-widget.fasl")
(#x35c208ff34764a5e412832f3ae9173ef . "./code.cg2js/cg-timer.fasl")
(#x680cdabb14962da25771094e2103de6b . "./code.cg2js/cg-toggling-widget.fasl")
(#x7c81c7676067e889365493f7d76ace71 . "./code.cg2js/cg-toolbar.fasl")
(#xf51d5a3cf96e366279dfc7d08ec76748 . "./code.cg2js/cg-tooltip.fasl")
(#xfb8a342d1e4d99f5646adb9b93669571 . "./code.cg2js/cg-touch.fasl")
(#x8a3c70f0e11f79b13af15af3e04dbce5 . "./code.cg2js/cg-trackbar.fasl")
(#x86bf66c16df3e7b6c01c3646044496c3 . "./code.cg2js/cg-tray.fasl")
(#xa012932bb489d9ea527e989ed1e23e32 . "./code.cg2js/cg-up-down-control.fasl")
(#x3cf2c6e76db9194888d9ad05b3ffe21e . "./code.cg2js/cg-utility-dialog.fasl")
(#xbc1a1f31da551a3bab0130b411409e00 . "./code.cg2js/cg-web-browser-dde.fasl")
(#x10474df748072f79dd3620021d60dcc7 . "./code.cg2js/cg-web-browser.fasl")
(#x31a6bd67ed5a21e0cc611e0f8de1c6ba . "./code.cg2js/cg-wrap-string.fasl")
(#xbf1d8c3b2625d3e94906e8d30940f6f8 . "./code.cg2js/cg-yes-no-list.fasl")
(#xb4f26b298a1e8a887ed14914ca3b8588 . "./code.cg2js/cg.fasl")
(#x3792a915e8417bdaccb2a4219dd98ef5 . "./code.cg2js/cg-yes-no-string.fasl")
(#xb8fdb33b65159b9db3459829a5d4b332 . "./code.idejs/ide-acache.fasl")
(#xd226083e2db0bc7d2ca130cd3b2aa77a . "./code.idejs/ide-apropos.fasl")
(#x4ad027ba2bcf11012a125a321a05248d . "./code.idejs/ide-base.fasl")
(#xfa37877ed18ca863eddc3b3c172aa14a . "./code.idejs/ide-class-browser.fasl")
(#x5162f3bcc2ea8830ed607809903bdc62 . "./code.idejs/ide-class-grapher.fasl")
(#xeb7178b56bf53d1ff0a0138b82fca633 . "./code.idejs/ide-clipboard.fasl")
(#x09d3916c96e520019cc1f71453c03d0a . "./code.idejs/ide-dev.fasl")
(#xd6a0bea4099f5fa1432212e0d9c2f1cc . "./code.idejs/ide-editor.fasl")
(#x38fa03f1d8e3c1d6f3b5b9840b43aaab . "./code.idejs/ide-find-definitions.fasl")
(#xeebf78ace61cb2c7974b357d3e1e79cf . "./code.idejs/ide-find-in-files.fasl")
(#xf1471414cb0a2a3137ce810106055df3 . "./code.idejs/ide-form.fasl")
(#xe12a63e3364c9b3b7247e0dba3623fa6 . "./code.idejs/ide-grapher.fasl")
(#xdbfa6e7f8d454739f1ce9747a6ed0d80 . "./code.idejs/ide-inspector.fasl")
(#x635a17af96f3845a0a646b0743df9feb . "./code.idejs/ide-intro.fasl")
(#x49e9d3e722fd22397ce2e0323675b7cd . "./code.idejs/ide-keyshort.fasl")
(#x1a019a4b7195476dca34a933960cb724 . "./code.idejs/ide-navigator.fasl")
(#xa53f9629123e5a3c23a1a99a7ee7667e . "./code.idejs/ide-package-list.fasl")
(#x53623f4e5fe037c8007cbe1745bf8072 . "./code.idejs/ide-patch-dialog.fasl")
(#xe3381047d465a57f327b0bad3df3a129 . "./code.idejs/ide-process-browser.fasl")
(#xf637d98853875b6bd98185ae9d31bdeb . "./code.idejs/ide-project.fasl")
(#xb74465d79619c7c5739d6754fbe0663f . "./code.idejs/ide-trace.fasl")
(#x3d393e968def5a20df20ff59f2c1c313 . "./code.idejs/ide-window-list.fasl")
(#x4598999fe923f5823e2f4625e0f770ad . "./code.idejs/ide.fasl")
(#xf954c7d1d43ea0e0cec9ada7b7316935 . "./cgjs.html")
(#x55fea08d92e113f5a8381c271febbc9e . "./ssl-not-found.html")
(#x4e1b21622d7d9cc4e6fc09c7875f8430 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
